// Author : Nicolas JEAN
// Date   : October 2001
// filename : if.c
// Copyright : Nicolas JEAN 2001 - 2004
// available on http://youpee.salemioche.com

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "if.h"
#include "common.h"
#include "define.h"
#include "youpee.h"

int file_exists(char *line);
int file_notexists(char *line);
int equal(char *line);
int notequal(char *line);
int isdef(char *line);
int isnotdef(char *line);
int gt(char *line);
int ge(char *line);
int lt(char *line);
int le(char *line);
int notnulllength(char *line);
int nulllength(char *line);

typedef struct { char *test;
                 int (*action)(char *line) ;
               } if_branch;

if_branch ifs[] = { { TEST_FILEEXISTS    , file_exists },
                    { TEST_FILENOTEXISTS , file_notexists },
                    { TEST_EQUAL         , equal },
                    { TEST_NOTEQUAL      , notequal},
                    { TEST_GT            , gt},
                    { TEST_GE            , ge},
                    { TEST_LT            , lt},
                    { TEST_LE            , le},
                    { TEST_Z             , nulllength},
                    { TEST_NOTZ          , notnulllength},
                    { TEST_DEF           , isdef},
                    { TEST_NOTDEF        , isnotdef},
                    { NULL               , NULL } };

int notnulllength(char *line) {
  return ! nulllength(line);
}

int nulllength(char *line) {
char *s = get_field(line, 3);
  if ( !s ) return TRUE;
  return !(strlen(s));
}

int isnotdef(char *line) {
  return ! isdef(line);
}

int isdef(char *line) {
char *s = get_field(line, 3);
  if ( !s ) return FALSE;
  if ( is_defined(s) ) 
    return TRUE;
  else {
    char *str; int rc;
    str = malloc( (strlen(s)+5)*sizeof(char));
    sprintf(str,BINDEF_STR,s);
    rc = is_bindefined(str);
    free(str);
    return rc;
  }
}

int file_notexists(char *line) {
  return ! file_exists(line);
}

int file_exists(char *line) {
char *env_include = getenv("INCLUDEPATH");
char *filename = get_field(line, 3), *fp;
FILE *f;

  if ( ( f = fopen(fp=findPath(filename,env_include), "r") ) ) {
    SHOW_DEP(fp);
    fclose(f);
    return TRUE;
  } else 
    return FALSE;
}

#define GET_TEST_S1S2(OP) \
long l1,l2;\
char *s1 = get_field(line,3);\
char *s2 = get_field(line,4);\
  if ( !s1 || !s2 ) return FALSE;\
  if ( is_long(s1,s2,&l1,&l2) ) return (l1 OP l2); else return strcmp(s1,s2) OP 0;

int is_long(char *s1, char *s2, long *l1, long *l2) {
char *p;
  *l1 = strtol(s1,&p,10);
  if ( p[0] ) return FALSE;
  *l2 = strtol(s2,&p,10);
  return ! p[0];
}

int gt(char *line) {
GET_TEST_S1S2(>);
}

int ge(char *line) {
GET_TEST_S1S2(>=);
}

int equal(char *line) {
GET_TEST_S1S2(==);
}

int lt(char *line) {
  return !ge(line);
}

int le(char *line) {
  return !gt(line);
}

int notequal(char *line) {
  return !equal(line);
}

void do_endif(FILE *in, FILE *out, char *line) {
  return;
}

void do_else(FILE *in, FILE *out, char *line) {
unsigned int level = 0;
char buf[MAX_LINE_LENGTH];
  while ( getstrline(buf, MAX_LINE_LENGTH, in) != NULL ) {
    if ( !strncmp_sol(buf, DIESE_IF) ) level++;
    if ( level ) {
      if ( !strncmp_sol(buf, DIESE_ENDIF) ) level--;
    }
    else
      if ( !strncmp_sol(buf, DIESE_ENDIF) ||
           !strncmp_sol(buf, DIESE_ELSE) ) 
          break;
  }
}


void do_if(FILE *in, FILE *out, char *line) {
unsigned int level = 0;
char buf[MAX_LINE_LENGTH];
char *test = get_field(line, 2);
if_branch *one_if = ifs;

  if ( ! test ) return;

  while ( one_if->test ) {
    if ( ! strncmp(one_if->test, test, strlen(one_if->test)) ) {
      if ( one_if->action(line) ) return;
      else break;
    }
    one_if++;
  }

  // test is false
  while ( getstrline(buf, MAX_LINE_LENGTH, in) != NULL ) {
    if ( !strncmp_sol(buf, DIESE_IF) ) level++;
    if ( level ) {
      if ( !strncmp_sol(buf, DIESE_ENDIF) ) level--;
    } else
      if ( !strncmp_sol(buf, DIESE_ENDIF) ||
           !strncmp_sol(buf, DIESE_ELSE ) ) 
           break;
  }
}
