// Author : Nicolas JEAN
// Date   : October 2001
// filename : image.c
// Copyright : Nicolas JEAN 2001 - 2004
// available on http://youpee.salemioche.com

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "image.h"
#include "common.h"

#define IMG_FREAD(b,l)  if ( fread(b, 1, l, in) != l ) return FALSE;

char *get_img_ext(char *fn) {
  if ( strlen(fn) <= 4 ) return NULL;
  return strlower(fn+strlen(fn)-3);
}

int jpg(char *filename, int *height, int *width) {
unsigned char buf[7], jfif[5];
FILE *in;
unsigned char w=0,h=0,w1=0,h1=0;
int len =0, offset;

        if((in = fopen(filename,"rb"))== NULL) return FALSE;
        SHOW_DEP(filename);
        IMG_FREAD(buf,4); // must be FF D8 FF E0
        IMG_FREAD(&w,1);
        IMG_FREAD(&w1,1);
        len = w*256 + w1;
        IMG_FREAD(jfif,5); // must be JFIF\0x00
       
        offset = 2;
        buf[0] = 0xFF; buf[1] = 0xFF;

        while ( buf[0] != 0xFF || !( buf[1] == 0xC0 || buf[1] == 0xC2 ) ) {
          offset += len +2;
          fseek(in, offset, SEEK_SET );
          IMG_FREAD(buf,2);
          IMG_FREAD(&w,1);
          IMG_FREAD(&w1,1);
          len = w*256 + w1;
        }
        IMG_FREAD(buf,1);
        IMG_FREAD(&h,1);
        IMG_FREAD(&h1,1);
        IMG_FREAD(&w,1);
        IMG_FREAD(&w1,1);
        *width = w*256+w1;
        *height = h*256+h1;
        fclose(in);     
        return TRUE;
}

char png_header[PNG_HEADER_LG] = { 0x89 ,0x50 ,0x4E ,0x47 ,0xD ,0xA ,0x1A ,0xA };
int png(char *filename, int *height, int *width) {
char buf[PNG_HEADER_LG];
FILE *in;
char devnull[8];
unsigned char w=0,w1=0,h=0,h1=0,w2=0,w3=0,h2=0,h3=0;

        if((in = fopen(filename,"rb"))== NULL) return FALSE;
        SHOW_DEP(filename);
        
        IMG_FREAD(buf,PNG_HEADER_LG);
        if ( memcmp(buf, png_header, PNG_HEADER_LG) ) return FALSE;        
        IMG_FREAD(devnull,8);
        
        IMG_FREAD(&w,1);
        IMG_FREAD(&w1,1);
        IMG_FREAD(&w2,1);
        IMG_FREAD(&w3,1);
        IMG_FREAD(&h,1);
        IMG_FREAD(&h1,1);
        IMG_FREAD(&h2,1);
        IMG_FREAD(&h3,1);
       
        *width = (w1+256*w)*256+w3+256*w2;
        *height = (h1+256*h)*256+h3+256*h2;

        fclose(in);     
        return TRUE;
}

int gif(char *filename, int *height, int *width) {
int rc = FALSE;
char buf[7];
FILE *in;
unsigned char w=0,h=0,w1=0,h1=0;

        if((in = fopen(filename,"rb"))== NULL) return FALSE;
        SHOW_DEP(filename);

        IMG_FREAD(buf,6); buf[6] = 0x00;
        IMG_FREAD(&w,1);
        IMG_FREAD(&w1,1);
        IMG_FREAD(&h,1);
        IMG_FREAD(&h1,1);

        if ( ! strncmp(buf,GIF89,strlen(GIF89)) || ! strncmp(buf,GIF87,strlen(GIF87))) {
          *width = w+256*w1 ;
          *height = h+256*h1;
          rc = TRUE;
        } else
          fprintf(stderr, __FILE__ ".%d: can't understand file %s\n",__LINE__,filename);
          
        fclose(in);     
        return rc;
}

int get_img_hw(char *name,int *height, int *width) 
{
char *imgpath = getenv(ENV_IMGPATH);
char *path = findPath(name, imgpath);
char *ext = get_img_ext(name);
int rc;
int h = 0 , w = 0;

    if ( !name ) return FALSE;

    if ( ! strncmp(ext, GIF_EXT, EXT_LG) ) 
      gif(path, &h, &w);
    else if ( ! strncmp(ext, PNG_EXT, EXT_LG) ) 
      png(path, &h, &w);
    else if ( ! strncmp(ext, JPG_EXT, EXT_LG) ) 
      jpg(path, &h, &w);
    else {
      fprintf(stderr, __FILE__ ".%d: unknown file extension %s (%s)\n",__LINE__,name,ext);
      return FALSE;
    }

    if ( w && h ) {
      *height = h ; 
      *width = w;
      rc = TRUE;
    } else
      rc = FALSE;

    free(ext);

    return rc;
}


char *do_img(char *field) 
{
char *buf;
char *name = get_field(field,IMG_NAME_FIELD);
char *attrib;
int h = 0 , w = 0;

    if ( !name ) return NULL;
    attrib = get_fromfield(field,IMG_NAME_FIELD+1);
    buf = malloc((strlen(IMG_FULL_STRING)+strlen(name)+6+6+strlen(attrib)+1)*sizeof(char));
    if ( get_img_hw(name, &h, &w) )
      if ( strstr(attrib, "alt") ) 
        sprintf(buf,  IMG_FULL_STRING, name, h, w, attrib);
      else
        sprintf(buf,  IMG_FULL_ALT_STRING, name, h, w, attrib);
    else {
      fprintf(stderr, __FILE__ ".%d: can't open file %s\n",__LINE__,name);
      sprintf(buf, IMG_SMALL_STRING, name, attrib);
    }
    return buf;
}


