// Author : Nicolas JEAN
// Date   : October 2001
// filename : script.c
// Copyright : Nicolas JEAN 2001 - 2004
// available on http://youpee.salemioche.com

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/unistd.h>
#include <sys/wait.h>
#include "youpee.h"
#include "script.h"
#include "common.h"


void do_exec(char *binary, char **argv, FILE *out) {
int pid;
int p[2];
FILE *pin;

  if ( pipe(p) ) {
    ERR("can't create pipe");
    exit(1);
  }

  switch ( pid = fork() ) {
   case -1:  fprintf(stderr, __FILE__ ".%d : can't fork %s\n", __LINE__ ,
                     DEFAULT_SCRIPT_FILENAME );
             return;
   case 0:  close(p[0]);
            if ( dup2(p[1], STDOUT_FILENO) == -1) { 
              perror("can't dup2 stdout : ");
              exit(1); 
            }
            execv(binary, argv);
            exit(1);
            break;
   default: close(p[1]);
            pin  = fdopen(p[0],"r");
            process_file(pin, out, NULL);
            fclose(pin); close(p[0]);
            break;
  }

}

void do_call(FILE *in, FILE *out, char *line) {
char *codepath = getenv(ENV_CODEPATH);
char *binary = findPath(get_field(line, 2), codepath);
char *argv[10] ;
int argc = 0;
    
  argv[argc++] = binary;
  SHOW_DEP(binary);
  while ( ( argv[argc] = get_field(line, argc+2) ) ) {
    if ( ! strncmp(argv[argc], DIESE_ENDCALL, strlen(DIESE_ENDCALL) )) {
      argv[argc] = NULL;
    }
    argc++;
  }

  do_exec(binary, argv, out);

}

void do_script(FILE *in, FILE *out, char *line) {
char buf[MAX_LINE_LENGTH];
char *codepath = getenv(ENV_CODEPATH);
char *shell = findPath(get_field(line, 2), codepath);
static int i=0;
int level=1;
char scriptfilename[] = DEFAULT_SCRIPT_FILENAME;
char *argv[10];
int argc=0;
int useScript=0;
FILE *scriptfile;
    
  argv[argc++] = shell;
  SHOW_DEP(shell);
  while ( ( argv[argc] = get_field(line, argc+2) ) ) {
    if ( !strncmp(argv[argc], DIESE_SCRIPTNAME, strlen(DIESE_SCRIPTNAME) )) {
      argv[argc]=scriptfilename;
      useScript=argc;
    } else
    if ( ! strncmp(argv[argc], ">", 1 )) {
      if (!useScript) argv[argc++]=scriptfilename;
      argv[argc] = NULL;
    }
    argc++;
  }
  do {
    i++;
    sprintf(scriptfilename,DEFAULT_SCRIPT_FILENAME, i);
    if ( ( scriptfile = fopen(scriptfilename,"r") ) ) { // file exists
      fclose(scriptfile);
      i++;
    } else
      break;
  } while ( i < MAX_PARALLEL_SCRIPT ) ;

  
  if ( ! ( scriptfile = fopen(scriptfilename, "w") ) ) {
    fprintf(stderr, __FILE__ ".%d : can't create %s\n", __LINE__ ,
                    scriptfilename );
    exit(1);
  }

  while ( getstrline(buf, MAX_LINE_LENGTH, in) != NULL ) {
    if ( ! strncmp_sol(buf, DIESE_SCRIPT ) ) level++;
    if ( ! strncmp_sol(buf, DIESE_ENDSCRIPT) ) {
      level--;
    }
    if ( level ) fprintf(scriptfile, buf); 
    else break;
  }
  fclose(scriptfile);
  
  do_exec(shell, argv, out);

  remove(scriptfilename);

}


void do_process(FILE *in, FILE *out, char *line) {
int pid;
int p[2];
FILE *pin, *pout;
unsigned int level = 0;
char buf[MAX_LINE_LENGTH];

  if ( pipe(p) ) {
    ERR("can't create pipe");
    exit(1);
  }

  switch ( pid = fork() ) {
   case -1:  fprintf(stderr, __FILE__ ".%d : can't fork %s\n", __LINE__ ,
                     DEFAULT_SCRIPT_FILENAME );
             return;
   case 0:  close(p[0]);
            pout  = fdopen(p[1],"w");
            process_file(in, pout, DIESE_ENDPROCESS);
            fclose(pout); close(p[1]);
            exit(1);
            break;
   default: close(p[1]);
            pin  = fdopen(p[0],"r");
            process_file(pin, out, NULL);
            fclose(pin); close(p[0]);
            while ( getstrline(buf, MAX_LINE_LENGTH, in) != NULL ) {
              if ( !strncmp_sol(buf, DIESE_PROCESS) ) level++;
              if ( level ) {
                if ( !strncmp_sol(buf, DIESE_ENDPROCESS) ) level--;
              }
              else
                if ( !strncmp_sol(buf, DIESE_ENDPROCESS) ) break;
            }
            break;
  }

}


